# 
#       sebastien.letort@toulouse.inra.fr
#       Created: september 18, 2007
#       Last Updated: september 18, 2007
#

package LipmError;

=pod

=head1 NAME

	LipmError - a class to interface Error classe developped in the lipm

=head1 SYNOPSIS

	# how to throw LipmError
	use LipmError;

	foo()	or throw LipmError( -text => $msg [, -debug => $dbg_msg ] );

	#how to catch LipmError
	use Error qw( :try );	# mandatory !
	use LipmError;

	try{ DangerousCode(); }
	catch LipmError with
	{
		my ( $err, $r ) = @_;
		print STDOUT $err->Explain(); # print $msg in a "Lipm" format
		print STDERR $err->Explain(1); # like above but can add information usefull for debugging.
	}
	otherwise
	{
		my ( $err, $r ) = @_;

		print STDERR $err->text();
	}
	; # do not forget it !

=head1 DESCRIPTION

	This class can be used to generate error but it is better to use one of its daugther.
	It centralized what is common with all Error class developped in the lipm.
	It declare our global error code and some common methods like how to print where the error is located.

=head1 SUBROUTINES

	GetMsg
	IsFatal
	Explain

=head1 AUTHOR

	Sebastien Letort : sebastien.letort@toulouse.inra.fr

=cut

use warnings;
use strict;

## this class inherit from Error.pm
use base qw(Error);

# constantes
use constant FATAL => 0001;

BEGIN
{
    our $VERSION = do {my @r = (q$Rev: 172 $ =~ /\d+/g); $r[0]};
}


=head2 Function IsFatal

	Title      :	IsFatal
	Usage      :	my $bool = $err->IsFatal()
	Prerequiste:    Fatal code are odd code value
	Function   :	tell if the program must stop
	Returns    :	a boolean, 1 if value is FATAL
	Args       :	none
	Error      :    none
	Globals    :    none

	TODO       :	use this beautifull &FATAL ;)

=cut

sub IsFatal
{
	my $self = shift;

	return 0	unless( defined( $self->{ -value } ) );

	return ($self->{ -value } & &FATAL);
}


=head2 Function GetMsg

	Title      :	GetMsg
	Usage      :	my $msg = $err->GetMsg()
	Prerequiste:    none
	Function   :	give a formatted msg containing where the error occurred
	Returns    :	a string
	Args       :	none
	Error      :    none
	Globals    :    none

=cut

sub GetMsg
{
	my $self = shift;

	# Construct message
	my $msg =  ref($self)." thrown by " . $self->{ -file };
	$msg .= " on line ". $self->{ -line } . "\n";

	return $msg;
}

=head2 function GetDebug

 Title        : GetDebug
 Usage        : $dbg_msg = $o_err->GetDebug();
 Prerequisite : none
 Function     : accessor for -debug attribute
 Returns      : $dbg_msg, a string, the -debug value
              	it returns undef if no -debug attribute was set.
 Args         : none
 Globals      : none

=cut

sub GetDebug
{
	my $self = shift;

	return $self->{ -debug }	if( defined( $self->{ -debug } ) );
	return;
}


=head2 Function Explain

	Title      :	Explain
	Usage      :	my $string = $err->Explain( $verbose )
	Prerequiste:    none
	Function   :	explain the error in details,
	            	give information for debugging if $verbose = 1
	Returns    :	a string, the error msg
	Args       :	$verbose, a boolean, true if you want information for debugging
	Error      :    none
	Globals    :    none

=cut

sub Explain
{
	my $self = shift;
	my ( $verbose ) = @_;

	my $msg = $self->GetMsg() . $self->text();

	return $msg	if( !defined( $verbose ) or ( 0 == $verbose ) );

	return	$msg . $self->GetDebug();
}

1;
